from typing import Any, Dict, Optional, Union

from sqlalchemy.orm import Session

from app.core.security import get_password_hash, verify_password
from app.models.user import User
from app.schemas.user import UserCreate, UserUpdate


def get_by_email(db: Session, email: str) -> Optional[User]:
    """
    Get a user by email
    """
    return db.query(User).filter(User.email == email).first()


def get_by_username(db: Session, username: str) -> Optional[User]:
    """
    Get a user by username
    """
    return db.query(User).filter(User.username == username).first()


def get_by_id(db: Session, id: int) -> Optional[User]:
    """
    Get a user by ID
    """
    return db.query(User).filter(User.id == id).first()


def get_by_email_or_username(
    db: Session, email: Optional[str] = None, username: Optional[str] = None
) -> Optional[User]:
    """
    Get a user by email or username
    """
    if email:
        return get_by_email(db, email)
    if username:
        return get_by_username(db, username)
    return None


def create(db: Session, *, obj_in: UserCreate) -> User:
    """
    Create a new user
    """
    db_obj = User(
        email=obj_in.email,
        username=obj_in.username,
        hashed_password=get_password_hash(obj_in.password),
        is_active=obj_in.is_active,
        is_superuser=obj_in.is_superuser,
    )
    db.add(db_obj)
    db.commit()
    db.refresh(db_obj)
    return db_obj


def update(
    db: Session, *, db_obj: User, obj_in: Union[UserUpdate, Dict[str, Any]]
) -> User:
    """
    Update a user
    """
    if isinstance(obj_in, dict):
        update_data = obj_in
    else:
        update_data = obj_in.dict(exclude_unset=True)

    # Handle password update
    if "password" in update_data:
        hashed_password = get_password_hash(update_data["password"])
        del update_data["password"]
        update_data["hashed_password"] = hashed_password

    for field in update_data:
        if hasattr(db_obj, field):
            setattr(db_obj, field, update_data[field])

    db.add(db_obj)
    db.commit()
    db.refresh(db_obj)
    return db_obj


def authenticate(
    db: Session, *, email: Optional[str] = None, username: Optional[str] = None, password: str
) -> Optional[User]:
    """
    Authenticate a user with email/username and password
    """
    user = get_by_email_or_username(db, email=email, username=username)
    if not user:
        return None
    if not verify_password(password, user.hashed_password):
        return None
    return user


def is_active(user: User) -> bool:
    """
    Check if a user is active
    """
    return user.is_active


def is_superuser(user: User) -> bool:
    """
    Check if a user is a superuser
    """
    return user.is_superuser
